#import "PDTVController.h"
#import "PDTVDoc.h"

@implementation PDTVController

- (id)init
{
	[super init];
	windowCount = 0;
	launched = NO;
	dragOn = NO;
	dropArray = [[NSMutableArray alloc] init];
	return self;
}

- (void)openFiles:(NSArray *)arr;
{
	id ret;
	BOOL success;
	int n = [arr count];
	int i;
	for(i=0;i<n;i++) {
		ret = [[PDTVDoc alloc] initWithFile:[arr objectAtIndex:i] owner:self count:windowCount++ success:&success];
		if(!success) {
			[ret release];
			windowCount--;
		}
		if(windowCount == 1) {
			[o_saveMenu setEnabled: YES];
			[o_closeMenu setEnabled: YES];
		}
	}
}

- (void)savePanelDidEnd:(NSSavePanel *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
	if(returnCode != NSOKButton) return;
	if([[[NSApp mainWindow] delegate] hasAlpha]) [[[NSApp mainWindow] delegate] exportPNG: [sheet filename] saveMode:[[o_imageType selectedItem] tag]];
	else [[[NSApp mainWindow] delegate] exportPNG: [sheet filename] saveMode:[[o_imageType2 selectedItem] tag]];
}

- (IBAction)exportPNG:(id)sender
{
	NSSavePanel *sp = [NSSavePanel savePanel];
	[sp setTitle: NSLocalizedString(@"Save As PNG...",@"")];
	if([[[NSApp mainWindow] delegate] hasAlpha]) [sp setAccessoryView: o_saveView];
	else [sp setAccessoryView: o_saveView2];
	NSString *defaultName = [[[[[NSApp mainWindow] delegate] filename] stringByDeletingPathExtension] stringByAppendingPathExtension:@"png"];
	[sp beginSheetForDirectory:nil file:defaultName modalForWindow:[NSApp mainWindow] modalDelegate:self didEndSelector:@selector(savePanelDidEnd:returnCode:contextInfo:)contextInfo: nil];
}

- (IBAction)openPDT:(id)sender
{
	NSOpenPanel *op;
	int stat;
	
	op = [NSOpenPanel openPanel];
	[op setTitle: NSLocalizedString(@"Open PDT...",@"")];
	[op setCanChooseDirectories: NO];
	[op setCanChooseFiles: YES];
	[op setAllowsMultipleSelection: YES];
	stat = [op runModalForDirectory: nil file: preveousSelected types: [NSArray arrayWithObjects:@"pdt", nil]];
							   
	if(stat != NSOKButton) return;
	if(preveousSelected) [preveousSelected release];
	preveousSelected = [[[op filename] lastPathComponent] retain];
	[self openFiles: [op filenames]];
}

- (IBAction)changeView:(id)sender
{
	int i;
	NSArray *arr = [o_channelSubMenu itemArray];
	for(i=0;i<3;i++) {
		[[arr objectAtIndex:i] setState: NSOffState];
	}
	[sender setState: NSOnState];
	if([sender tag] == 0) [[[NSApp mainWindow] delegate] changeToRGBA];
	else if([sender tag] == 1) [[[NSApp mainWindow] delegate] changeToRGB];
	else [[[NSApp mainWindow] delegate] changeToAlpha];
}

- (IBAction)toggleScale:(id)sender
{
	if([sender state] == NSOnState) [sender setState: NSOffState];
	else [sender setState: NSOnState];
}

- (void)windowDidClose:(id)view
{
	windowCount--;
	if(!windowCount) {
		[o_saveMenu setEnabled: NO];
		[o_closeMenu setEnabled: NO];
		[o_channelMenu setEnabled: NO];
	}
	[view release];
}

- (void)setEnableChangeView:(BOOL)flag select:(int)mode
{
	[o_channelMenu setEnabled: flag];
	if(flag) {
		int i;
		NSArray *arr = [o_channelSubMenu itemArray];
		for(i=0;i<3;i++) {
			[[arr objectAtIndex:i] setState: NSOffState];
		}
		[[arr objectAtIndex:mode] setState: NSOnState];
	}
}

- (BOOL)doesScale
{
	if([o_scaleMenu state] == NSOnState) return YES;
	return NO;
}

- (void)dealloc
{
	if(preveousSelected) [preveousSelected release];
	[dropArray release];
	[super dealloc];
}

- (void)acceptDrop
{
	if(![dropArray count]) return;
	[self openFiles: [dropArray sortedArrayUsingSelector:@selector(caseInsensitiveCompare:)]];
	[dropArray removeAllObjects];
	dragOn = NO;
}

- (void)applicationDidFinishLaunching: (NSNotification *)notification
{
	[o_saveMenu setEnabled: NO];
	[o_closeMenu setEnabled: NO];
	[o_channelMenu setEnabled: NO];
	[self acceptDrop];
	launched = YES;
}

- (BOOL)application:(NSApplication *)theApplication openFile:(NSString *)filename
{
	[dropArray addObject:filename];
	if(launched) {
		if(!dragOn) {
			[NSTimer scheduledTimerWithTimeInterval:0.3
				target:self
				selector:@selector(acceptDrop)
				userInfo:nil
				repeats:NO];
			dragOn = YES;
		}
	}
	
	return YES;
}
@end
